%
% function [X_check, Psi] = CPPCA_Decoder(Y_tilde, P, L)
%
%   This function implements the CPPCA decoder.
%  
%   Y_tilde is length-J cell array of projected data, and P is a
%   length-J cell array of N x K projection matrices. L is a scalar
%   indicating the number of principal components to recover.
%
%

%
% CPPCA: Compressive-Projection Principal Component Analysis
% Copyright (C) 2007-2014  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function [X_check, Psi] = CPPCA_Decoder(Y_tilde, P, L)

J = length(Y_tilde);
[N K] = size(P{1});

for j = 1:J
  U{j} = P{j} * PCA_Train(Y_tilde{j});
end

Psi = GetEigenvectors(U, P, L);
X_check = GetCoefficients(Y_tilde, P, Psi);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function Psi = GetEigenvectors(U, P, L)

J = length(U);
[N K] = size(P{1});

W_prime = zeros(N, L);

for l = 1:L

  QQ = [];
  initial_vector = zeros(N, 1);

  for j = 1:J
    Q = [U{j}(:, l) null(P{j}')];
    QQ = [QQ Q * Q'];
    initial_vector = initial_vector + U{j}(:, l);
  end
  initial_vector = initial_vector / J;

  W_prime(:, l) = POCS(QQ, initial_vector);
  W_prime(:, l) = W_prime(:, l) / norm(W_prime(:, l));
end

Psi = W_prime;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function w_hat = POCS(QQ, initial_vector)

max_iterations = 100;
delta1 = 0.001;

N = size(QQ, 1);
J = size(QQ, 2) / N;

w_hat_previous = initial_vector;

for i = 1:max_iterations
  w_hat = QQ * repmat(w_hat_previous, [J 1]) / J;
  
  s = SAM(w_hat, w_hat_previous);
  if (s > 90);
    s = 180 - s;
  end
  if (s < delta1)
    return;
  end

  w_hat_previous = w_hat;
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function X_check = GetCoefficients(Y_tilde, P, Psi)

J = length(Y_tilde);

L = size(Psi, 2);

for j = 1:J
%   disp(['    j = ' num2str(j)]);
  
  [K Mj] = size(Y_tilde{j});
  [N K] = size(P{j});
  X_check{j} = zeros(N, Mj);

  X_check{j}(1:L,:) = pinv(P{j}' * Psi) * Y_tilde{j};
end
