%
% function X_check = CS_Decoder(Y_tilde, P, Psi, epsilon)
%
%   This function implements fixed-basis compressed-sensing (CS)
%   decoding.
%
%   In order to perform basis-pursuit l1 optimization,
%   this code relies on routines in l1-magic (http://www.l1-magic.org)
%   which will need to be installed within the current MATLAB
%   search path (see the "addpath" command below).
%  
%   Y_tilde is length-J cell array of projected data, and P is a
%   length-J cell array of N x K projection matrices. Psi is the
%   N x N fixed-basis transform matrix. epsilon is error bound for
%   the quadratic error constraint (see (11)).
%
%   Coefficient recovery uses quadratically relaxed basis pursuit (BP).
%

%
% CPPCA: Compressive-Projection Principal Component Analysis
% Copyright (C) 2007-2014  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function X_check = CS_Decoder(Y_tilde, P, Psi, epsilon)

% Redirect to local installation of l1magic if needed
% addpath(genpath('/usr/local/share/l1magic'));

if (nargin == 3)
  epsilon = 0.01;
end

J = length(Y_tilde);

for j = 1:J
  
  [K Mj] = size(Y_tilde{j});
  [N K] = size(P{j});
  X_check{j} = zeros(N, Mj);

  for m = 1:Mj
    X_check{j}(:, m) = BP(Y_tilde{j}(:, m), P{j}, Psi, epsilon);
  end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function x_check = BP(y_tilde, P, Psi, epsilon)

if (exist('l1qc_logbarrier') ~= 2)
  error('l1-magic (http://www.l1-magic.org) must be installed in the current MATLAB search path.');
end

x_check = Psi' * P * y_tilde;

if (sum(abs(x_check)) < 3e-16)
  return;
end

x_check = l1qc_logbarrier(x_check, P'*Psi, [], y_tilde, ...
    epsilon, 1e-4, 2, 1e-8, 200);
