%
% function X_check = MTBCS_Decoder(Y_tilde, P, Psi, Gamma_Factor)
%
%   This function implements fixed-basis multitask compressed-sensing (MTCS)
%   decoding.
%
%   In order to perform MTCS decoding, this code relies on
%   routines in the Bayesian Compressed Sensing package
%   (http://people.ee.duke.edu/~lihan/cs/) which will need to be
%   installed within the current MATLAB search path.
%
%  
%   Y_tilde is length-J cell array of projected data, and P is a
%   length-J cell array of N x K projection matrices. Psi is the
%   N x N fixed-basis transform matrix. epsilon is error bound for
%   the quadratic error constraint (see (11)). Gamma_Factor is an
%   optional scaling factor for the a parameter to the gamma prior
%   in the MT-CS algorithm (default = 100).
%
%   See:
%     S. Ji, D. Dunson, and L. Carin, "Multitask Compressive Sensing" 
%       IEEE Trans. Signal Processing, vol. 57, pp. 92-106, January 2009.
%

%
% CPPCA: Compressive-Projection Principal Component Analysis
% Copyright (C) 2007-2009  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function X_check = MTBCS_Decoder(Y_tilde, P, Psi, Gamma_Factor)

% Redirect to local installation of mt_bcs if needed
% addpath(genpath('../BCS/MT_CS_demo'));

if (nargin < 4)
  Gamma_Factor = 100;
end

J = length(Y_tilde);

Phi = {};
V = {};
sigma = [];
Mjs = [];
N2 = size(Psi, 2);

for j = 1:J
  
  [K Mj] = size(Y_tilde{j});
  [N K] = size(P{j});
  Mjs = [Mjs Mj];
  
  Phi = [Phi mat2cell(repmat(P{j}' * Psi, [1 Mj]), K, ones(1, Mj) * N2)];
  V = [V mat2cell(Y_tilde{j}, K, ones(1, Mj))];
  sigma = [sigma std(Y_tilde{j})];
end

a = Gamma_Factor / mean(sigma).^2;
b = 1;
X_check = mat2cell(mt_CS(Phi, V, a, b, 1e-8), N2, Mjs);

