%
% function Run_TIP_Fig10()
%
%   This function reproduces results from the paper:
%     J. E. Fowler, "Compressive-Projection Principal Component
%       Analysis," submitted to IEEE Transactions on Image Processing,
%       June 2008. Revised February 2009.
%  
%   Specifically, this function produces Figs. 10 and 11.
%  
%   Note: since this code relies on randomly generated
%   projection matrices, your results may differ slightly
%   in appearance and value from those in the paper.
%

%
% CPPCA: Compressive-Projection Principal Component Analysis
% Copyright (C) 2007-2014  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function Run_TIP_Fig10()

addpath(genpath('../BCS/MT_CS_demo'));

M = 10000;
filename = 'cuprite';
Ls =[4 3 3 8 14 20];
% filename = 'moffett';
% Ls = [5 10 5 10 15 30];
% filename = 'jasper';
% Ls = [ 5 4 7 7 20 30];

fid = fopen([filename '.dat'], 'r', 'ieee-be');
X = fread(fid, [224 M], 'float32');
fclose(fid);

[N M] = size(X);
J = 20;

projection_matrix_file = ['projections.' num2str(N) '.' num2str(J) '.mat'];

Psi_PCA = PCA_Train(X);

lifting_scheme = liftwave('haar');
num_levels = 5;
Psi_Haar = DWT_Matrix(N, lifting_scheme, num_levels);

lifting_scheme = liftwave('db4');
num_levels = 5;
Psi_D4 = DWT_Matrix(N, lifting_scheme, num_levels);

Psi_DCT = DCT_Matrix(N);

% for m = 1:M
%   x = X(:, m);
%   x_check = Psi_PCA'*x;
%   x_check((L + 1):N) = 0;
%   D_PCA(m) = SNR(x, Psi_PCA*x_check);
% end
% 
% disp('SNR for true PCA:')
% disp(['  ' num2str(mean(D_PCA)) ' (dB)']);

relative_dimensions = [0.1 0.15 0.2 0.3 0.4 0.5];
Ks = round(relative_dimensions * N);

X_original = X;

disp('Running CPPCA...');
for index1 = 1:length(Ks)
  
  K = Ks(index1);
  L = Ls(index1);
  
  P{index1} = CPPCA_GenerateProjections(N, K, J, projection_matrix_file);

  [Y_tilde{index1}, X] = CPPCA_Encoder(X_original, P{index1});
  tic
  [X_check_CPPCA, Psi_CPPCA] = CPPCA_Decoder(Y_tilde{index1}, P{index1}, L);
  toc
  
  D_CPPCA(index1) = SNR_Dataset(X, X_check_CPPCA, ...
      [Psi_CPPCA zeros(N, N - L)]);

end

% disp(['  CPPCA: ' num2str(D_CPPCA) ' dB']);

disp('Running MT-BCS-D4...');
for index1 = 1:length(Ks)
  tic
  X_check_MTBCSD4 = MTBCS_Decoder(Y_tilde{index1}, P{index1}, Psi_D4);
  toc
  D_MTBCSD4(index1) = SNR_Dataset(X, X_check_MTBCSD4, Psi_D4);
end

% disp(['  MT-BCS-D4: ' num2str(D_MTBCSD4) ' dB']);

disp('Running MT-BCS-DCT...');
for index1 = 1:length(Ks)
  X_check_MTBCSDCT = MTBCS_Decoder(Y_tilde{index1}, P{index1}, Psi_DCT);
  D_MTBCSDCT(index1) = SNR_Dataset(X, X_check_MTBCSDCT, Psi_DCT);
end

% disp(['  MT-BCS-DCT: ' num2str(D_MTBCSDCT) ' dB']);

disp('Running MT-BCS-Haar...');
for index1 = 1:length(Ks)
  X_check_MTBCSHaar = MTBCS_Decoder(Y_tilde{index1}, P{index1}, Psi_Haar);
  D_MTBCSHaar(index1) = SNR_Dataset(X, X_check_MTBCSHaar, Psi_Haar);
end

% disp(['  MT-BCS-Haar: ' num2str(D_MTBCSHaar) ' dB']);

% save([filename '.snr.results.mat'], ...
%       'relative_dimensions', ...
%       'D_CPPCA', 'D_MTBCSDCT', 'D_MTBCSHaar', 'D_MTBCSD4');
% 
% load([filename '.snr.results.mat']);

D_CPPCA = D_CPPCA(3:length(D_CPPCA));
D_MTBCSD4 = D_MTBCSD4(3:length(D_MTBCSD4));
D_MTBCSDCT = D_MTBCSDCT(3:length(D_MTBCSDCT));
D_MTBCSHaar = D_MTBCSHaar(3:length(D_MTBCSHaar));
relative_dimensions = ...
    relative_dimensions(3:length(relative_dimensions));

figure(1);
clf;
plot(relative_dimensions, D_CPPCA, 'k-', 'LineWidth', 2);
hold on
grid on
xlabel('K/N');
ylabel('Average SNR (dB)');

plot(relative_dimensions, D_MTBCSD4, 'k-x', 'LineWidth', 2);

plot(relative_dimensions, D_MTBCSDCT, 'k--', 'LineWidth', 2);

plot(relative_dimensions, D_MTBCSHaar, 'k-.', 'LineWidth', 2);

legend('CPPCA', 'MT-BCS-D4', 'MT-BCS-DCT', 'MT-BCS-Haar', ...
    'Location', 'SouthEast');
