%
% function S = SAM(x, x_hat)
%
%   This function calculates the angle, in degrees, between
%   x and x_hat. SAM denotes "spectral angle mapper."
%
%   If x and x_hat are both vectors, the SAM between the two vectors
%   is returned; S is a scalar.
%
%   If x and x_hat are both matrices, then a row vector S is returned
%   wherein each entry in S gives the SAM between the corresponding
%   columns in x and x_hat.
%

%
% CPPCA: Compressive-Projection Principal Component Analysis
% Copyright (C) 2007-2014  James E. Fowler
% http://www.ece.mstate.edu/~fowler
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
%


function S = SAM(x, x_hat)

if (isvector(x) && isvector(x_hat))
  if (length(x) ~= length(x_hat))
    error('Vector x and x_hat must both have the same length');
  end
  x = x(:);
  x_hat = x_hat(:);
else
  if (~isvector(x) && ~isvector(x_hat))
    if (size(x) ~= size(x_hat))
      error('Sizes of matrices x and x_hat must be the same');
    end
  else
    error('x and x_hat must be either both vectors or both matrices');
  end
end

S = zeros(1, size(x, 2));

x_norm = sqrt(sum(x .^ 2));
x_hat_norm = sqrt(sum(x_hat .^ 2));

I = ((x_norm ~= 0) & (x_hat_norm ~= 0));
S(I) = real(acos(sum(x(:, I) .* x_hat(:, I)) ./ x_norm(I) ./ ...
    x_hat_norm(I))) / pi * 180;

